import { Injectable, NgModule } from '@angular/core';
import { ApolloLink, Observable } from 'apollo-link';
import { BehaviorSubject, Subject, asyncScheduler, pipe } from 'rxjs';
import { observeOn, distinctUntilChanged, share } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 * @return {?}
 */
function event() {
    return pipe(share(), observeOn(asyncScheduler));
}
class ApolloNetworkStatus extends ApolloLink {
    constructor() {
        super();
        this.count = 0;
        this.pending$ = new BehaviorSubject(false);
        this.request$ = new Subject();
        this.success$ = new Subject();
        this.error$ = new Subject();
        this.cancel$ = new Subject();
        this.isPending = this.pending$.asObservable().pipe(event(), distinctUntilChanged());
        this.onRequest = this.request$.asObservable().pipe(event());
        this.onSuccess = this.success$.asObservable().pipe(event());
        this.onError = this.error$.asObservable().pipe(event());
        this.onCancel = this.cancel$.asObservable().pipe(event());
    }
    /**
     * @param {?} operation
     * @param {?} forward
     * @return {?}
     */
    request(operation, forward) {
        this._onRequest({ operation });
        /** @type {?} */
        const subscriber = forward(operation);
        return new Observable(observer => {
            /** @type {?} */
            let isPending = true;
            /** @type {?} */
            const subscription = subscriber.subscribe({
                next: result => {
                    isPending = false;
                    this._onSuccess({ operation, result });
                    observer.next(result);
                },
                error: error => {
                    isPending = false;
                    this._onError({ operation, error });
                    observer.error(error);
                },
            });
            return () => {
                if (isPending)
                    this._onCancel({ operation });
                if (subscription)
                    subscription.unsubscribe();
            };
        });
    }
    /**
     * @param {?} data
     * @return {?}
     */
    _onRequest(data) {
        this.request$.next(data);
        this.increase();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    _onSuccess(data) {
        this.success$.next(data);
        this.decrease();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    _onError(data) {
        this.error$.next(data);
        this.decrease();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    _onCancel(data) {
        this.cancel$.next(data);
        this.decrease();
    }
    /**
     * @return {?}
     */
    increase() {
        this.count++;
        this.update();
    }
    /**
     * @return {?}
     */
    decrease() {
        this.count--;
        this.update();
    }
    /**
     * @return {?}
     */
    update() {
        this.pending$.next(this.count > 0);
    }
}
ApolloNetworkStatus.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ApolloNetworkStatus.ctorParameters = () => [];
class ApolloNetworkStatusModule {
}
ApolloNetworkStatusModule.decorators = [
    { type: NgModule, args: [{
                providers: [ApolloNetworkStatus],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { event, ApolloNetworkStatus, ApolloNetworkStatusModule };

//# sourceMappingURL=ng.apollo-network-status.js.map