import { __extends } from 'tslib';
import { Injectable, NgModule } from '@angular/core';
import { ApolloLink, Observable } from 'apollo-link';
import { BehaviorSubject, Subject, asyncScheduler, pipe } from 'rxjs';
import { observeOn, distinctUntilChanged, share } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 * @return {?}
 */
function event() {
    return pipe(share(), observeOn(asyncScheduler));
}
var ApolloNetworkStatus = /** @class */ (function (_super) {
    __extends(ApolloNetworkStatus, _super);
    function ApolloNetworkStatus() {
        var _this = _super.call(this) || this;
        _this.count = 0;
        _this.pending$ = new BehaviorSubject(false);
        _this.request$ = new Subject();
        _this.success$ = new Subject();
        _this.error$ = new Subject();
        _this.cancel$ = new Subject();
        _this.isPending = _this.pending$.asObservable().pipe(event(), distinctUntilChanged());
        _this.onRequest = _this.request$.asObservable().pipe(event());
        _this.onSuccess = _this.success$.asObservable().pipe(event());
        _this.onError = _this.error$.asObservable().pipe(event());
        _this.onCancel = _this.cancel$.asObservable().pipe(event());
        return _this;
    }
    /**
     * @param {?} operation
     * @param {?} forward
     * @return {?}
     */
    ApolloNetworkStatus.prototype.request = /**
     * @param {?} operation
     * @param {?} forward
     * @return {?}
     */
    function (operation, forward) {
        var _this = this;
        this._onRequest({ operation: operation });
        /** @type {?} */
        var subscriber = forward(operation);
        return new Observable(function (observer) {
            /** @type {?} */
            var isPending = true;
            /** @type {?} */
            var subscription = subscriber.subscribe({
                next: function (result) {
                    isPending = false;
                    _this._onSuccess({ operation: operation, result: result });
                    observer.next(result);
                },
                error: function (error) {
                    isPending = false;
                    _this._onError({ operation: operation, error: error });
                    observer.error(error);
                },
            });
            return function () {
                if (isPending)
                    _this._onCancel({ operation: operation });
                if (subscription)
                    subscription.unsubscribe();
            };
        });
    };
    /**
     * @param {?} data
     * @return {?}
     */
    ApolloNetworkStatus.prototype._onRequest = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        this.request$.next(data);
        this.increase();
    };
    /**
     * @param {?} data
     * @return {?}
     */
    ApolloNetworkStatus.prototype._onSuccess = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        this.success$.next(data);
        this.decrease();
    };
    /**
     * @param {?} data
     * @return {?}
     */
    ApolloNetworkStatus.prototype._onError = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        this.error$.next(data);
        this.decrease();
    };
    /**
     * @param {?} data
     * @return {?}
     */
    ApolloNetworkStatus.prototype._onCancel = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        this.cancel$.next(data);
        this.decrease();
    };
    /**
     * @return {?}
     */
    ApolloNetworkStatus.prototype.increase = /**
     * @return {?}
     */
    function () {
        this.count++;
        this.update();
    };
    /**
     * @return {?}
     */
    ApolloNetworkStatus.prototype.decrease = /**
     * @return {?}
     */
    function () {
        this.count--;
        this.update();
    };
    /**
     * @return {?}
     */
    ApolloNetworkStatus.prototype.update = /**
     * @return {?}
     */
    function () {
        this.pending$.next(this.count > 0);
    };
    ApolloNetworkStatus.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ApolloNetworkStatus.ctorParameters = function () { return []; };
    return ApolloNetworkStatus;
}(ApolloLink));
var ApolloNetworkStatusModule = /** @class */ (function () {
    function ApolloNetworkStatusModule() {
    }
    ApolloNetworkStatusModule.decorators = [
        { type: NgModule, args: [{
                    providers: [ApolloNetworkStatus],
                },] }
    ];
    return ApolloNetworkStatusModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { event, ApolloNetworkStatus, ApolloNetworkStatusModule };

//# sourceMappingURL=ng.apollo-network-status.js.map